
///////////////////////////////////////////////////
// define LCD control function

#define INSTRUCTION 	0x00
#define DATA 			0x01
#define Function_set 	0x38
#define Display_clear 	0x01
#define Display_on 		0x0C
#define Display_off 	0x08
#define Entry_mode 		0x06
#define First_line		0x00
#define Second_line		0xc0
#define Cursor_home 	0x02
#define Busy_flag 		0x80

// LCD Control
#define LCD_DATA_DDR		DDRA
#define LCD_DATA_PORTo		PORTA
#define LCD_DATA_PORTi		PINA
#define LCD_CONTROL_PORT	PORTG
#define LCD_EN		PG2
#define LCD_WR		PG1
#define LCD_RS		PG0

#define   RS_ON		sbi(LCD_CONTROL_PORT, LCD_RS)
#define   RS_OFF	cbi(LCD_CONTROL_PORT, LCD_RS)
#define   WR_ON   	sbi(LCD_CONTROL_PORT, LCD_WR)
#define   WR_OFF  	cbi(LCD_CONTROL_PORT, LCD_WR)
#define   E_ON   	sbi(LCD_CONTROL_PORT, LCD_EN)
#define   E_OFF  	cbi(LCD_CONTROL_PORT, LCD_EN)

#define   LCD_READ  LCD_DATA_PORTi
#define   LCD_WRITE(inst)  (LCD_DATA_PORTo=inst)//outp(inst,PORTA)


void data_in(void)
{
	LCD_DATA_DDR=0x00;
	LCD_DATA_PORTo=0xFF;
}

void data_out(void)
{
	LCD_DATA_DDR=0xFF;
}


void busy_check(void)
{
	BYTE temp1=0;
	data_in();
	RS_OFF;
	WR_ON;
	do {
		E_ON;
		_delay_us(1);
		temp1 = LCD_READ;
		E_OFF;
	} while (temp1&Busy_flag);
	data_out();
}

void Write_Inst(BYTE inst)
{
	busy_check();
	RS_OFF; // instruction
	WR_OFF;
	
	_delay_us(1);
	LCD_WRITE(inst);
	_delay_us(1);
	E_ON;
	_delay_us(1);
	E_OFF;
}


void Write_Data(BYTE data)
{
	busy_check();
	RS_ON; // data
	WR_OFF;
	
	_delay_us(1);
	LCD_WRITE(data);
	_delay_us(1);
	E_ON;
	_delay_us(1);
	E_OFF;
    _delay_us(100);
}

void Write_Str(BYTE line, BYTE add, char *strn)
{
    if(line == 0) Write_Inst(0x80 + add);
    else if (line == 1) Write_Inst(0xC0 + add);
    _delay_us(100);

    while(*strn) Write_Data(*strn++);
}


void Init_LCD(void)
{
	RS_OFF;
	WR_OFF;
	E_OFF;

	_delay_ms(2);
    Write_Inst(Function_set);

    _delay_ms(2);
    Write_Inst(Function_set);

    _delay_us(40);
    Write_Inst(Function_set);
    _delay_us(40);

    Write_Inst(Function_set);
    _delay_us(40);

    Write_Inst(Display_off);
    _delay_us(40);

    Write_Inst(Display_clear);
    _delay_ms(2);

    Write_Inst(Entry_mode);
    _delay_us(40);

    Write_Inst(Cursor_home);
    _delay_ms(2);


    Write_Inst(Display_on);
    _delay_us(40);
}
