/*************************************
Programmed by US Technology
http://www.us-technology.co.kr
*************************************/

#define F_CPU	16000000

#include <avr/interrupt.h>
#include <avr/io.h>
#include <avr/eeprom.h>
#include <avr/wdt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <stdlib.h>
#include <string.h>

#include "defines.h"
#include "variables.h"


// UART
volatile BYTE GPS_Buffer[200];
volatile BYTE GPS_index[20];
volatile WORD uart1_index=0;
volatile BYTE flag_analyze_data=0;

void USART_Transmit(BYTE port, BYTE data);

// Timer
volatile WORD Count=0;
volatile BYTE Timer1_Count=0;
volatile BYTE flag_loop_hold = 0;


#include "LCD.h"
#include "gps.h"

volatile BYTE flag_msg_index=0;
volatile BYTE flag_msg_mindex=0;

SIGNAL(SIG_UART0_RECV)
{
	BYTE RS_Char;
	
	RS_Char = UDR0;
}

SIGNAL(SIG_UART1_RECV)
{
	BYTE RS_Char;
	char str[200];

	RS_Char = UDR1;
	
	if (RS_Char == '$') 
	{
		flag_msg_index=0;
		uart1_index=0;
		GPS_index[flag_msg_index]=uart1_index;
	}
	else if ((RS_Char == ',')||(RS_Char == '*'))
	{
		flag_msg_index++;
		GPS_index[flag_msg_index]=uart1_index+1;
	}
	else if (RS_Char == 0x0A)
	{
		flag_analyze_data=1;
		flag_msg_index=0;
		uart1_index=0;
	}
	
	GPS_Buffer[uart1_index++]=RS_Char;
	
	if (flag_analyze_data==1)
	{
		memset(str, 0, sizeof(str));
		memcpy(str, GPS_Buffer+1, 5);
		
		switch (protocol)
		{
			case GPRMC:
				if (memcmp(str+2,"RMC",3)==0)
				{
					// time
					memset(str, 0, sizeof(str));
					memcpy(str, GPS_Buffer+GPS_index[1], GPS_index[2]-GPS_index[1]-1);
					gps_msg.UTC_Time = atol(str);
					
					// status
					if (GPS_Buffer[GPS_index[2]]=='A') gps_msg.status = GPS_VALID;
					else gps_msg.status = GPS_INVALID;
					
					// latitude
					memset(str, 0, sizeof(str));
					memcpy(str, GPS_Buffer+GPS_index[3], GPS_index[4]-GPS_index[3]-1);
					gps_msg.latitude = atof(str);
					
					// longitude
					memset(str, 0, sizeof(str));
					memcpy(str, GPS_Buffer+GPS_index[5], GPS_index[6]-GPS_index[5]-1);
					gps_msg.longitude = atof(str);
					
					// speed
					memset(str, 0, sizeof(str));
					memcpy(str, GPS_Buffer+GPS_index[7], GPS_index[8]-GPS_index[7]-1);
					gps_msg.speed = 1.852*atof(str);
				}
				break;
		}
		flag_analyze_data=0;
	}
	
}

SIGNAL(SIG_OVERFLOW0) // 10mS
{
	char str0[16], str1[16];
	
	TCNT0 = 177;

	if ((Count%100)==0)
	{
		switch (protocol)
		{
			case GPRMC:
				sprintf(str0, "RMC: %06ld    ", gps_msg.UTC_Time);
				sprintf(str1, "%05d %05d %03d ", (long)(gps_msg.latitude), (long)(gps_msg.longitude), (int)(gps_msg.speed));
				break;
		}
		Write_Str(0, 0, str0);
		Write_Str(1, 0, str1);
		if (gps_msg.status==GPS_INVALID) Write_Str(0, 15, "*");
	}

	Count++;
	if (Count>=60000) Count=0;
}


void Init_UART0(unsigned long BaudRate)
{
	UCSR0A = 0x00;          // multi_communication
	//UCSR0B = 0xF8;          // 0b 1101 1100 RXCIE,TXCIE,UDRIE,RxEN,TxEN,xxx
	UCSR0B = 0xD8;
    UCSR0C = 0x06;          // 0000 0110

    UBRR0H = 0x00;         // Setting BaudRate
    UBRR0L = (int)(F_CPU/BaudRate/16 - 1);
}

void Init_UART1(unsigned long BaudRate)
{
	UCSR1A = 0x00;          // multi_communication
	UCSR1B = 0xD8;
    UCSR1C = 0x06;          // 0000 0110

    UBRR1H = 0x00;         // Setting BaudRate
    UBRR1L = (int)(F_CPU/BaudRate/16 - 1);
}

void Init_Port(void)
{
	DDRA=0xff;
	PORTA=0x00;
	
	DDRG=0xff;
	PORTG=0x00;
}

void Init_Timer0(void) // global timer
{
    TCCR0 = 7;				// 1/64 
   	TCNT0 = 177;				// 

   	sbi(TIMSK, TOIE0);
}


void Init_ADC(BYTE channel)
{
	int i;
	for (i=0; i<5; i++) ADC_Data[i]=0;
	
	//Single Mode
	ADMUX = 0x20|channel;
	ADCSRA = 0x88;
}

int main(void)
{
	Init_Port();
	Init_Timer0();
	//Init_ADC(0);
	Init_UART0(9600);
	Init_UART1(9600);
	
	
	Init_LCD();

//	Write_Strn(0, 0, "  US Technology  ");
//	Write_Strn(1, 0, "    LCD TEST     ");
	sei();
	
	
	while(1)
	{
	}
	return 0;
}
